@@ -138,7 +138,7 @@ def grouped(iterable: Iterable, n: int, *, keep_tail=False) -> List[tuple]:
         return [tuple(batch) for batch in manager]
 
 
-def is_numeric(s: str) -> bool:
+def is_numeric(s: Any) -> bool:
     """ Check whether `s` is a number. """
 
     try:
@@ -173,7 +173,8 @@ def get_one_hot(feature: Union[list, np.ndarray], dim: int) -> np.ndarray:
 
 
 def show_or_save(export_path: str,
-                 fig: plt.figure = None, **kwargs) -> None:
+                 fig: plt.figure = None,
+                 **kwargs) -> None:
     """
     Utility function to deal with figure.
 
@@ -486,7 +487,7 @@ class LoggingMixin:
         if self._logging_path_ is None:
             folder = os.path.join(os.getcwd(), "_logging", type(self).__name__)
             os.makedirs(folder, exist_ok=True)
-            self._logging_path_ = os.path.join(folder, f"{timestamp()}.log")
+            self._logging_path_ = self.generate_logging_path(folder)
         return self._logging_path_
 
     @property
@@ -515,6 +516,10 @@ class LoggingMixin:
             handler.close()
             logger.removeHandler(handler)
 
+    @staticmethod
+    def generate_logging_path(folder: str) -> str:
+        return os.path.join(folder, f"{timestamp()}.log")
+
     def _init_logging(self,
                       verbose_level: Union[int, None] = 2,
                       trigger: bool = True):
@@ -624,14 +629,19 @@ class LoggingMixin:
 
     @classmethod
     def start_timer(cls, name):
+        if name in cls._time_cache_dict_:
+            print(f"{cls.warning_prefix}'{name}' was already in time cache dict, "
+                  "this may cause by calling `start_timer` repeatedly")
+            return
         cls._time_cache_dict_[name] = time.time()
 
     @classmethod
     def end_timer(cls, name):
         start_time = cls._time_cache_dict_.pop(name, None)
         if start_time is None:
-            raise KeyError(f"'{name}' was not found in time cache dict, "
-                           "this may cause by not calling `start_timer` method")
+            print(f"{cls.warning_prefix}'{name}' was not found in time cache dict, "
+                  "this may cause by not calling `start_timer` method")
+            return
         incrementer = cls._timing_dict_.setdefault(name, Incrementer())
         incrementer.update(time.time() - start_time)
 
@@ -803,10 +813,15 @@ class SavingMixin(LoggingMixin):
             return False
         return verbose_level >= 5
 
-    def _data_tuple_context(self, *, is_saving):
+    def _data_tuple_context(self,
+                            *,
+                            is_saving: bool):
         return data_tuple_saving_controller(self, is_saving=is_saving)
 
-    def save(self, folder: str, *, compress: bool = True):
+    def save(self,
+             folder: str,
+             *,
+             compress: bool = True):
         with self._data_tuple_context(is_saving=True):
             Saving.save_instance(self, folder, self.log_msg)
         if compress:
@@ -816,10 +831,13 @@ class SavingMixin(LoggingMixin):
                 Saving.compress(abs_folder, remove_original=True)
         return self
 
-    def load(self, folder: str, *, compress: bool = True):
+    def load(self,
+             folder: str,
+             *,
+             compress: bool = True):
         base_folder = os.path.dirname(os.path.abspath(folder))
         with lock_manager(base_folder, [folder]):
-            with Saving.compress_loader(self, folder, compress, remove_extracted=True):
+            with Saving.compress_loader(folder, compress, remove_extracted=True, logging_mixin=self):
                 with self._data_tuple_context(is_saving=False):
                     Saving.load_instance(self, folder, log_method=self.log_msg)
         return self
@@ -1044,15 +1062,20 @@ class Saving(LoggingMixin):
             shutil.rmtree(abs_folder)
 
     @staticmethod
-    def compress_loader(instance, folder, is_compress, *, remove_extracted=True):
+    def compress_loader(folder: str,
+                        is_compress: bool,
+                        *,
+                        remove_extracted: bool = True,
+                        logging_mixin: LoggingMixin = None):
         class _manager(context_error_handler):
             def __enter__(self):
                 if is_compress:
                     if os.path.isdir(folder):
-                        instance.log_msg(
-                            f"'{folder}' already exists, it will be cleared up to load our model",
-                            instance.warning_prefix, msg_level=logging.WARNING
-                        )
+                        msg = f"'{folder}' already exists, it will be cleared up to load our model"
+                        if logging_mixin is None:
+                            print(msg)
+                        else:
+                            logging_mixin.log_msg(msg, logging_mixin.warning_prefix, msg_level=logging.WARNING)
                         shutil.rmtree(folder)
                     with zipfile.ZipFile(f"{folder}.zip", "r") as zip_ref:
                         zip_ref.extractall(folder)
@@ -1583,7 +1606,9 @@ class timing_context(context_error_handler):
 
     """
 
-    def __init__(self, logging_mixin, name):
+    def __init__(self,
+                 logging_mixin: LoggingMixin,
+                 name: str):
         self._cls, self._name = logging_mixin, name
 
     @property
