@@ -9,7 +9,7 @@ def register_metric(name, sign, requires_prob):
 
 
 __all__ = [
-    "Anneal", "Metrics", "ScalarEMA", "Visualizer",
+    "Anneal", "Metrics", "ScalarEMA", "Visualizer", "Tracker",
     "Estimator", "ModelPattern", "EnsemblePattern", "Comparer",
     "register_metric"
 ]
