@@ -1,5 +1,6 @@
 import os
 import math
+import shutil
 import logging
 
 import numpy as np
@@ -282,6 +283,11 @@ collate_fn_type = Callable[[List[np.ndarray], bool], np.ndarray]
 predict_method_type = Union[estimate_fn_type, None]
 
 
+class Statistics(NamedTuple):
+    msg: str
+    data: Dict[str, Dict[str, float]]
+
+
 class Estimator(LoggingMixin):
     """
     Util class to estimate the performances of a group of methods, on specific dataset & metric.
@@ -290,7 +296,7 @@ class Estimator(LoggingMixin):
     ----------
     metric_type : str, indicates which kind of metric is to be calculated.
     verbose_level : int, verbose level used in `LoggingMixin`.
-    **kwargs : used to initialize `Metrics` instance.
+    metric_config : used to initialize `Metrics` instance.
 
     Examples
     --------
@@ -314,10 +320,10 @@ class Estimator(LoggingMixin):
                  metric_type: str,
                  *,
                  verbose_level: int = 2,
-                 **kwargs):
+                 metric_config: Dict[str, Any] = None):
         self._reset()
         self._verbose_level = verbose_level
-        self._metric = Metrics(metric_type, **kwargs)
+        self._metric = Metrics(metric_type, metric_config)
 
     def __str__(self):
         return f"Estimator({self.type})"
@@ -348,27 +354,14 @@ class Estimator(LoggingMixin):
 
     # API
 
-    def estimate(self,
-                 x: np.ndarray,
-                 y: np.ndarray,
-                 methods: Dict[str, Union[estimate_fn_type, List[estimate_fn_type]]],
-                 *,
-                 scoring_function: Union[str, scoring_fn_type] = "default",
-                 verbose_level: int = 1) -> Dict[str, Dict[str, float]]:
-        self._reset()
-        if isinstance(scoring_function, str):
-            scoring_function = getattr(self, f"_{scoring_function}_scoring")
-        for k, v in methods.items():
-            if not isinstance(v, list):
-                methods[k] = [v]
-        self.raw_metrics = {
-            name: np.array([self._metric.metric(y, method(x)) for method in sub_methods], np.float32)
-            for name, sub_methods in methods.items()
-        }
+    def get_statistics(self,
+                       scoring_function: Union[str, scoring_fn_type] = "default") -> Statistics:
         msg_list = []
         statistics = {}
         best_idx, best_score = -1, -math.inf
         sorted_method_names = sorted(self.raw_metrics)
+        if isinstance(scoring_function, str):
+            scoring_function = getattr(self, f"_{scoring_function}_scoring")
         for i, name in enumerate(sorted_method_names):
             raw_metrics = self.raw_metrics[name]
             mean, std = raw_metrics.mean().item(), raw_metrics.std().item()
@@ -385,8 +378,47 @@ class Estimator(LoggingMixin):
         width = max(map(len, msg_list))
         msg_list.insert(0, "=" * width)
         msg_list.append("-" * width)
-        self.log_block_msg("\n".join(msg_list), self.info_prefix, "Results", verbose_level)
-        return statistics
+        return Statistics("\n".join(msg_list), statistics)
+
+    def estimate(self,
+                 x: np.ndarray,
+                 y: np.ndarray,
+                 methods: Dict[str, Union[estimate_fn_type, List[estimate_fn_type]]],
+                 *,
+                 scoring_function: Union[str, scoring_fn_type] = "default",
+                 verbose_level: int = 1) -> Dict[str, Dict[str, float]]:
+        self._reset()
+        for k, v in methods.items():
+            if not isinstance(v, list):
+                methods[k] = [v]
+        self.raw_metrics = {
+            name: np.array([self._metric.metric(y, method(x)) for method in sub_methods], np.float32)
+            for name, sub_methods in methods.items()
+        }
+        statistics = self.get_statistics(scoring_function)
+        self.log_block_msg(statistics.msg, self.info_prefix, "Results", verbose_level)
+        return statistics.data
+
+    @classmethod
+    def merge(cls, estimators: List["Estimator"]) -> "Estimator":
+        first_estimator = estimators[0]
+        new_final_scores = {key: [value] for key, value in first_estimator.final_scores.items()}
+        new_raw_metrics = {key: [value] for key, value in first_estimator.raw_metrics.items()}
+        for estimator in estimators[1:]:
+            for key, value in estimator.raw_metrics.items():
+                new_raw_metrics[key].append(value)
+                new_final_scores[key].append(estimator.final_scores[key])
+        new_raw_metrics = {k: np.concatenate(v) for k, v in new_raw_metrics.items()}
+        new_final_scores = {k: sum(v) / len(v) for k, v in new_final_scores.items()}
+        first_metric_ins = first_estimator._metric
+        new_estimator = cls(
+            first_metric_ins.type,
+            verbose_level=first_estimator._verbose_level,
+            metric_config=first_metric_ins.config
+        )
+        new_estimator.raw_metrics = new_raw_metrics
+        new_estimator.final_scores = new_final_scores
+        return new_estimator
 
 
 class PatternBase(ABC):
@@ -468,11 +500,6 @@ class ModelPattern(PatternBase, LoggingMixin):
             if self.model is None:
                 raise ValueError("Either init_method or Callable predict_method is required in ModelPattern")
             predict_method = getattr(self.model, predict_method, None)
-        elif self.model is not None:
-            self.log_msg(
-                "predict_method is Callable but model is also created, which has no effect",
-                self.warning_prefix, 2, logging.WARNING
-            )
         return predict_method
 
     @classmethod
@@ -633,9 +660,9 @@ class Comparer(LoggingMixin):
     def best_methods(self) -> Dict[str, str]:
         return {k: v.best_method for k, v in self.estimators.items()}
 
-    def _log_statistics(self,
-                        verbose_level: int,
-                        **kwargs) -> None:
+    def log_statistics(self,
+                       verbose_level: int = 1,
+                       **kwargs) -> None:
         sorted_metrics = sorted(self.estimator_statistics)
         body = {}
         same_choices: choices_type = None
@@ -654,37 +681,40 @@ class Comparer(LoggingMixin):
                 same_choices = [None] * len(sub_header)
                 best_choices = [None] * len(sub_header)
             for method_idx, method in enumerate(sorted_methods):
-                method_statistics = statistics[method]
-                method_statistics = [method_statistics[stat_type] for stat_type in statistic_types]
-                body.setdefault(method, []).extend(method_statistics)
-                if best_choices is not None:
-                    for statistic_idx, method_statistic in enumerate(method_statistics):
-                        choice_idx = metric_idx * len(statistic_types) + statistic_idx
-                        current_idx_choice = best_choices[choice_idx]
-                        if current_idx_choice is None:
-                            best_choices[choice_idx] = method_idx
-                        else:
-                            stat_type = statistic_types[statistic_idx]
-                            chosen_stat = statistics[sorted_methods[current_idx_choice]][stat_type]
-                            if method_statistic == chosen_stat:
-                                if same_choices[choice_idx] is None:
-                                    same_choices[choice_idx] = {method_idx}
-                                else:
-                                    same_choices[choice_idx].add(method_idx)
-                            elif stat_type == "std":
-                                if method_statistic < chosen_stat:
-                                    same_choices[choice_idx] = None
-                                    best_choices[choice_idx] = method_idx
-                            elif stat_type == "score":
-                                if method_statistic > chosen_stat:
-                                    same_choices[choice_idx] = None
-                                    best_choices[choice_idx] = method_idx
+                method_statistics = statistics.get(method)
+                if method_statistics is None:
+                    method_statistics = [math.nan for _ in statistic_types]
+                else:
+                    method_statistics = [method_statistics[stat_type] for stat_type in statistic_types]
+                    if best_choices is not None:
+                        for statistic_idx, method_statistic in enumerate(method_statistics):
+                            choice_idx = metric_idx * len(statistic_types) + statistic_idx
+                            current_idx_choice = best_choices[choice_idx]
+                            if current_idx_choice is None:
+                                best_choices[choice_idx] = method_idx
                             else:
-                                assert stat_type == "mean"
-                                sign = Metrics.sign_dict[metric_type]
-                                if method_statistic * sign > chosen_stat * sign:
-                                    same_choices[choice_idx] = None
-                                    best_choices[choice_idx] = method_idx
+                                stat_type = statistic_types[statistic_idx]
+                                chosen_stat = statistics[sorted_methods[current_idx_choice]][stat_type]
+                                if method_statistic == chosen_stat:
+                                    if same_choices[choice_idx] is None:
+                                        same_choices[choice_idx] = {method_idx}
+                                    else:
+                                        same_choices[choice_idx].add(method_idx)
+                                elif stat_type == "std":
+                                    if method_statistic < chosen_stat:
+                                        same_choices[choice_idx] = None
+                                        best_choices[choice_idx] = method_idx
+                                elif stat_type == "score":
+                                    if method_statistic > chosen_stat:
+                                        same_choices[choice_idx] = None
+                                        best_choices[choice_idx] = method_idx
+                                else:
+                                    assert stat_type == "mean"
+                                    sign = Metrics.sign_dict[metric_type]
+                                    if method_statistic * sign > chosen_stat * sign:
+                                        same_choices[choice_idx] = None
+                                        best_choices[choice_idx] = method_idx
+                body.setdefault(method, []).extend(method_statistics)
         padding = 2 * (kwargs.get("padding", 1) + 3)
         method_length = kwargs.get("method_length", 16) + padding
         float_length = kwargs.get("float_length", 8)
@@ -772,11 +802,30 @@ class Comparer(LoggingMixin):
             self.estimator_statistics[estimator.type] = estimator.estimate(
                 x, y, methods,
                 scoring_function=scoring_function,
-                verbose_level=verbose_level + 5
+                verbose_level=None if verbose_level is None else verbose_level + 5
             )
-        self._log_statistics(verbose_level, **kwargs)
+        self.log_statistics(verbose_level, **kwargs)
         return self
 
+    @classmethod
+    def merge(cls,
+              comparer_list: List["Comparer"],
+              scoring_function: Union[str, scoring_fn_type] = "default") -> "Comparer":
+        first_comparer = comparer_list[0]
+        new_estimators = [
+            Estimator.merge([comparer.estimators[k] for comparer in comparer_list])
+            for k in first_comparer.estimators.keys()
+        ]
+        new_comparer = cls(
+            first_comparer.patterns, new_estimators,
+            verbose_level=first_comparer._verbose_level
+        )
+        new_comparer.estimator_statistics = {
+            estimator.type: estimator.get_statistics(scoring_function).data
+            for estimator in new_estimators
+        }
+        return new_comparer
+
 
 class ScalarEMA:
     """
@@ -1013,8 +1062,226 @@ class Visualizer:
         return show_or_return(return_canvas)
 
 
+class Tracker:
+    """
+    Util class to track records in an experiment.
+    * Currently only scalars are supported.
+
+    Parameters
+    ----------
+    project_name : {str, None}, the project name of the experiment.
+    * If None, then `Tracker.default_project_name()` will be used.
+    task_name : {str, None}, the task name of the experiment.
+    * If None, then `timestamp(ensure_different=True)` will be used.
+    base_folder : {str, None}, where the records will be stored.
+    * If None, then `Tracker.default_base_folder()` will be used.
+    overwrite : bool, whether overwrite the existing records.
+    * If False (which is by default), `Tracker` will load the existing records.
+
+    Examples
+    --------
+    >>> from cftool.ml.utils import Tracker
+    >>>
+    >>> tracker = Tracker()
+    >>> tracker.track_scalar("acc", 0.34)
+    >>> tracker.track_scalar("acc", 0.67)
+    >>> tracker.visualize_scalars()
+
+    """
+
+    def __init__(self,
+                 project_name: str = None,
+                 task_name: str = None,
+                 *,
+                 base_folder: str = None,
+                 overwrite: bool = False):
+        if base_folder is None:
+            base_folder = self.default_base_folder()
+        if project_name is None:
+            project_name = self.default_project_name()
+        if task_name is None:
+            task_name = timestamp(ensure_different=True)
+        self.base_folder = base_folder
+        self.project_name = project_name
+        self.task_name = task_name
+        self.project_folder = os.path.join(base_folder, project_name)
+        self.log_folder = os.path.join(self.project_folder, task_name)
+        exists = os.path.isdir(self.log_folder)
+        if exists:
+            if not overwrite:
+                print(f"loading tracker from '{self.log_folder}'")
+                self._load()
+            else:
+                print(f"{LoggingMixin.warning_prefix}'{self.log_folder}' already exists, it will be overwritten")
+                shutil.rmtree(self.log_folder)
+        if not exists or overwrite:
+            os.makedirs(self.log_folder)
+            self.reset()
+
+    def __str__(self):
+        return f"Tracker(project={self.project_name}, task={self.task_name})"
+
+    __repr__ = __str__
+
+    # core
+
+    @property
+    def scalars_folder(self) -> str:
+        folder = os.path.join(self.log_folder, "scalars")
+        os.makedirs(folder, exist_ok=True)
+        return folder
+
+    @staticmethod
+    def default_base_folder() -> str:
+        home = os.path.expanduser("~")
+        return os.path.join(home, ".carefree-toolkit", ".tracker")
+
+    @staticmethod
+    def default_project_name() -> str:
+        cwd = os.getcwd()
+        return f"{hash_code(cwd)}_{os.path.split(cwd)[-1]}"
+
+    def _load(self) -> None:
+        self._load_scalars()
+
+    def _load_scalars(self) -> None:
+        self.scalars = {}
+        for file in os.listdir(self.scalars_folder):
+            data = []
+            name = os.path.splitext(file)[0]
+            with open(os.path.join(self.scalars_folder, file), "r") as f:
+                for line in f:
+                    data.append(float(line.strip() or "nan"))
+            self.scalars[name] = data
+
+    # api
+
+    def reset(self) -> None:
+        self.scalars: Dict[str, List[float]] = {}
+
+    def track_scalar(self,
+                     name: str,
+                     value: float) -> None:
+        file = os.path.join(self.scalars_folder, f"{name}.txt")
+        data = self.scalars.setdefault(name, [])
+        data.append(value)
+        if not os.path.isfile(file):
+            with open(file, "w") as f:
+                f.write("\n".join(map(str, data)))
+        else:
+            with open(file, "a") as f:
+                f.write(f"\n{'' if math.isnan(value) else value}")
+
+    def visualize_scalars(self,
+                          types: List[str] = None,
+                          *,
+                          export_folder: str = None,
+                          merge: bool = True) -> None:
+        if export_folder is not None:
+            os.makedirs(export_folder, exist_ok=True)
+        if types is None:
+            types = list(self.scalars.keys())
+        plt.figure()
+        for i, name in enumerate(sorted(types)):
+            data = self.scalars[name]
+            plt.plot(np.arange(len(data)), data, label=name)
+            if not merge:
+                plt.legend()
+                export_path = None if export_folder is None else os.path.join(export_folder, f"{name}.png")
+                show_or_save(export_path)
+                if i != len(types) - 1:
+                    plt.figure()
+        if merge:
+            plt.legend()
+            export_path = None if export_folder is None else os.path.join(export_folder, "merged.png")
+            show_or_save(export_path)
+
+    # clear
+
+    @staticmethod
+    def _confirm(confirm: bool) -> bool:
+        if not confirm:
+            return True
+        if input("[Y/n] (default : n)").lower() != "y":
+            print("canceled")
+            return False
+        return True
+
+    def clear(self,
+              *,
+              confirm: bool = True) -> None:
+        print(f"clearing '{self.log_folder}'")
+        if self._confirm(confirm):
+            shutil.rmtree(self.log_folder)
+
+    def clear_project(self,
+                      *,
+                      confirm: bool = True) -> None:
+        print(f"clearing '{self.project_folder}'")
+        if self._confirm(confirm):
+            shutil.rmtree(self.project_folder)
+
+    def clear_all(self,
+                  *,
+                  confirm: bool = True) -> None:
+        print(f"clearing '{self.base_folder}'")
+        if self._confirm(confirm):
+            shutil.rmtree(self.base_folder)
+
+    # class methods
+
+    @classmethod
+    def compare(cls,
+                project_name: str = None,
+                task_names: List[str] = None,
+                *,
+                base_folder: str = None,
+                visualize: bool = True,
+                types: List[str] = None,
+                export_folder: str = None,
+                merge: bool = False) -> List["Tracker"]:
+        if base_folder is None:
+            base_folder = Tracker.default_base_folder()
+        if project_name is None:
+            project_name = Tracker.default_project_name()
+        project_folder = os.path.join(base_folder, project_name)
+        if task_names is None:
+            task_names = os.listdir(project_folder)
+        else:
+            for task_name in task_names:
+                task_folder = os.path.join(project_folder, task_name)
+                if not os.path.isdir(task_folder):
+                    raise ValueError(f"'{task_folder}' does not exist")
+        trackers = [
+            Tracker(project_name, task_name, base_folder=base_folder)
+            for task_name in task_names
+        ]
+        if visualize:
+            plt.figure()
+            if types is None:
+                types = set()
+                for tracker in trackers:
+                    types |= set(tracker.scalars.keys())
+                types = list(types)
+            for i, name in enumerate(sorted(types)):
+                for task_name, tracker in zip(task_names, trackers):
+                    data = tracker.scalars.get(name, [])
+                    plt.plot(np.arange(len(data)), data, label=f"{name} - {task_name}")
+                if not merge:
+                    plt.legend()
+                    export_path = None if export_folder is None else os.path.join(export_folder, f"{name}.png")
+                    show_or_save(export_path)
+                    if i != len(types) - 1:
+                        plt.figure()
+            if merge:
+                plt.legend()
+                export_path = None if export_folder is None else os.path.join(export_folder, "merged.png")
+                show_or_save(export_path)
+        return trackers
+
+
 __all__ = [
-    "Anneal", "Metrics", "ScalarEMA", "Visualizer",
+    "Anneal", "Metrics", "ScalarEMA", "Visualizer", "Tracker",
     "Estimator", "ModelPattern", "EnsemblePattern", "Comparer",
     "collate_fn_type", "estimate_fn_type", "scoring_fn_type",
     "pattern_type", "patterns_type",
